"use strict";
var Rx_1 = require('rxjs/Rx');
var serialport = require('serialport');
var ayb = require('all-your-base');
var config_1 = require('./config');
var WaterRower = (function () {
    function WaterRower(options) {
        var _this = this;
        this.data$ = new Rx_1.Subject();
        this.distance_l = 0;
        this.distance_h = 0;
        this.strokeRate = 0;
        this.speed_l = 0;
        this.speed_h = 0;
        this.clock = 0;
        var portName = options.portName || config_1.default.portName;
        var baudRate = options.baudRate || config_1.default.baudRate || 19200;
        var refreshRate = options.refreshRate || config_1.default.refreshRate || 200;
        if (!portName)
            throw "A port name is required";
        // setup the serial port
        this.port = new serialport.SerialPort(portName, {
            baudrate: baudRate,
            disconnectedCallback: function () { console.log('disconnected'); },
            parser: serialport.parsers.readline("\n")
        });
        // when the port opens, initialize it
        this.port.on('open', function () {
            console.log("A connection to the WaterRower has been established on " + portName);
            this.initialize(); //start things off
            this.setDisplayUnits(); //change the display to meters
            this.reset(); //reset the waterrower 
            setInterval(function () {
                this.requestDistance();
                this.requestSpeed();
                this.requestClock();
            }, refreshRate);
        });
        //when data comes in from the serial port, push it into our Rx Subject
        this.port.on('data', function (d) { return _this.data$.next(d); });
        //when a message is received, apply the appropriate action
        this.data$.subscribe(function (d) {
            _this.actions.forEach(function (a) {
                var matches = a.pattern.exec(d);
                if (matches && a.action)
                    a.action(matches);
            });
        });
        this.port.on('closed', function () { return console.log('connection closed'); });
        this.port.on('error', function (err) { return console.log('Please plug in your WaterRower and start again...'); });
        // this is the declarative list of things to do when the
        // waterrower sends us a data message
        // it uses regular expressions to respond to certain message patterns
        // m is the matches array and m[1] is the value of the primary regex match
        this.actions = [
            {
                name: 'distance (low byte)',
                pattern: /IDS055([\dA-F]+)/,
                action: function (m) {
                    if (_this.distance_l != m[1]) {
                        _this.distance_l = m[1];
                        _this.data$.next(null);
                    }
                }
            },
            {
                name: 'distance (high byte)',
                pattern: /IDS056([\dA-F]+)/,
                action: function (m) { return _this.distance_h = m[1]; }
            },
            {
                name: 'speed (low byte)',
                pattern: /IDS14A([\dA-F]+)/,
                action: function (m) {
                    if (_this.speed_l != m[1])
                        _this.speed_l = m[1];
                }
            },
            {
                name: 'speed (high byte)',
                pattern: /IDS14B([\dA-F]+)/,
                action: function (m) {
                    if (_this.speed_h != m[1]) {
                        _this.speed_h = m[1];
                        _this.data$.next(null);
                    }
                }
            },
            {
                name: 'stroke rate',
                pattern: /IDS142([\dA-F]+)/,
                action: function (m) {
                    if (_this.strokeRate != m[1]) {
                        _this.strokeRate = m[1];
                        _this.data$.next(null);
                    }
                }
            },
            {
                name: 'clock (seconds)',
                pattern: /IDS1E1([\dA-F]+)/,
                action: function (m) {
                    if (_this.clock != m[1]) {
                        _this.clock = m[1];
                        _this.data$.next(null);
                    }
                }
            },
            {
                name: 'pulse',
                pattern: /P(\d+)/,
                action: null
            }
        ];
    }
    /// initialize the connection    
    WaterRower.prototype.initialize = function () {
        this.send('USB');
    };
    /// send a serial message
    WaterRower.prototype.send = function (value) {
        this.port.write(value + '\r\n');
    };
    Object.defineProperty(WaterRower.prototype, "data", {
        /// get current data
        get: function () {
            return {
                distance: ayb.hexToDec(this.distance_h + '' + this.distance_l),
                strokeRate: ayb.hexToDec(this.strokeRate.toString()),
                speed: ayb.hexToDec(this.speed_h + '' + this.speed_l),
                clock: ayb.hexToDec(this.clock.toString()),
            };
        },
        enumerable: true,
        configurable: true
    });
    /// reset console
    WaterRower.prototype.reset = function () {
        this.send('RESET'); //reset the waterrower 
    };
    /// set up new workout session on the WR with set distance
    WaterRower.prototype.startRace = function (options) {
        this.send('WSI1' + ayb.decToHex(options.distance));
    };
    /// request distance data
    WaterRower.prototype.requestDistance = function () {
        this.send('IRS055'); //low byte of distance (m)
        this.send('IRS056'); //hi byte of distance (m)
    };
    /// request speed data
    WaterRower.prototype.requestSpeed = function () {
        this.send('IRS14A'); //low byte of average speed (m/s)
        this.send('IRS14B'); //hi byte of average speed (m/s)
    };
    /// request clock data
    WaterRower.prototype.requestClock = function () {
        this.send('IRS1E1'); //clock seconds
    };
    /// change the display to meters
    WaterRower.prototype.setDisplayUnits = function () {
        this.send('DDME');
    };
    return WaterRower;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = WaterRower;
//# sourceMappingURL=index.js.map